/*
**    TP CPE Lyon
**    Copyright (C) 2014 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <Qt/QtGui>
#include "window.hpp"
#include "glwidget.hpp"

#include <ui_mainwindow.h>

#include <iostream>

Window::Window(QWidget *parent)
  :QMainWindow(parent),ui(new Ui::MainWindow)
{
  ui->setupUi(this);
  glWidget = new glwidget;
  ui->layout_scene->addWidget(glWidget);

  connect(ui->actionExit,SIGNAL(triggered()),this,SLOT(quit_window()));
  connect(ui->wireframe_checkbox,SIGNAL(stateChanged(int)),this,SLOT(set_wireframe(int)));

  connect(ui->grid_checkbox,SIGNAL(clicked()),this,SLOT(grid_checkbox_clicked()));
  connect(ui->gridposition_checkbox,SIGNAL(clicked()),this,SLOT(gridposition_checkbox_clicked()));

  connect(ui->surface_checkbox,SIGNAL(clicked()),this,SLOT(surface_checkbox_clicked()));
}

Window::~Window()
{
  if(ui!=0)
    {
      delete ui;
      ui=0;
    }
}

void Window::quit_window()
{
  std::cout<<"Exit window"<<std::endl;
  this->window()->close();
}

void Window::set_wireframe(int is_active)
{
    if(is_active==0)
        glWidget->set_filled();
    else
        glWidget->set_wireframe();
}


void Window::grid_checkbox_clicked()
{
    const bool is_clicked=ui->grid_checkbox->isChecked();
    glWidget->current_scene.gui_visible_interface().set_grid_visible(is_clicked);
}

void Window::gridposition_checkbox_clicked()
{
    const bool is_clicked=ui->gridposition_checkbox->isChecked();
    glWidget->current_scene.gui_visible_interface().set_gridposition_visible(is_clicked);
}







void Window::surface_checkbox_clicked()
{
    const bool is_clicked=ui->surface_checkbox->isChecked();
    glWidget->current_scene.gui_visible_interface().set_surface_visible(is_clicked);
}
