/*
**    TP CPE Lyon
**    Copyright (C) 2014 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/



#include "patch4.hpp"
#include "../lib3d/matrix4.hpp"

namespace cpe
{
patch4::patch4()
{
    for(unsigned int k=0;k<16;++k)
        internal_data[k]=vec3(0.0,0.0,0.0);
}

patch4::patch4(const vec3& x00,const vec3& x01,const vec3& x02,const vec3& x03,
               const vec3& x10,const vec3& x11,const vec3& x12,const vec3& x13,
               const vec3& x20,const vec3& x21,const vec3& x22,const vec3& x23,
               const vec3& x30,const vec3& x31,const vec3& x32,const vec3& x33)
{
    internal_data[0]=x00; internal_data[4]=x01; internal_data[8 ]=x02; internal_data[12]=x03;
    internal_data[1]=x10; internal_data[5]=x11; internal_data[9 ]=x12; internal_data[13]=x13;
    internal_data[2]=x20; internal_data[6]=x21; internal_data[10]=x22; internal_data[14]=x23;
    internal_data[3]=x30; internal_data[7]=x31; internal_data[11]=x32; internal_data[15]=x33;
}

const vec3& patch4::operator()(unsigned int ku ,unsigned int kv) const
{
    assert_size(ku,kv);
    return internal_data[ku+4*kv];
}
vec3& patch4::operator()(unsigned int ku ,unsigned int kv)
{
    assert_size(ku,kv);
    return internal_data[ku+4*kv];
}

matrix4 patch4::get_x() const
{
    return matrix4(internal_data[0].x(),internal_data[4].x(),internal_data[8 ].x(),internal_data[12].x(),
                   internal_data[1].x(),internal_data[5].x(),internal_data[9 ].x(),internal_data[13].x(),
                   internal_data[2].x(),internal_data[6].x(),internal_data[10].x(),internal_data[14].x(),
                   internal_data[3].x(),internal_data[7].x(),internal_data[11].x(),internal_data[15].x());
}
matrix4 patch4::get_y() const
{
    return matrix4(internal_data[0].y(),internal_data[4].y(),internal_data[8 ].y(),internal_data[12].y(),
                   internal_data[1].y(),internal_data[5].y(),internal_data[9 ].y(),internal_data[13].y(),
                   internal_data[2].y(),internal_data[6].y(),internal_data[10].y(),internal_data[14].y(),
                   internal_data[3].y(),internal_data[7].y(),internal_data[11].y(),internal_data[15].y());
}
matrix4 patch4::get_z() const
{
    return matrix4(internal_data[0].z(),internal_data[4].z(),internal_data[8 ].z(),internal_data[12].z(),
                   internal_data[1].z(),internal_data[5].z(),internal_data[9 ].z(),internal_data[13].z(),
                   internal_data[2].z(),internal_data[6].z(),internal_data[10].z(),internal_data[14].z(),
                   internal_data[3].z(),internal_data[7].z(),internal_data[11].z(),internal_data[15].z());
}
void patch4::assert_size(unsigned int ku,unsigned int kv) const
{
    if(ku>3 || kv>3)
        throw std::string("Error")+__PRETTY_FUNCTION__;
}
}
