/*
**    TP CPE Lyon
**    Copyright (C) 2014 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "quad_index.hpp"
#include <string>
#include "../libinterface/string_helper.hpp"

namespace cpe
{
quad_index::quad_index()
{
    for(unsigned int k=0;k<4;++k)
        internal_u[k]=0;
}
quad_index::quad_index(unsigned int u0,unsigned int u1,unsigned int u2,unsigned int u3)
{
    internal_u[0]=u0;
    internal_u[1]=u1;
    internal_u[2]=u2;
    internal_u[3]=u3;
}


unsigned int quad_index::u0() const {return internal_u[0];}
unsigned int quad_index::u1() const {return internal_u[1];}
unsigned int quad_index::u2() const {return internal_u[2];}
unsigned int quad_index::u3() const {return internal_u[3];}
unsigned int& quad_index::u0() {return internal_u[0];}
unsigned int& quad_index::u1() {return internal_u[1];}
unsigned int& quad_index::u2() {return internal_u[2];}
unsigned int& quad_index::u3() {return internal_u[3];}
unsigned int quad_index::operator[](unsigned int k_index) const
{
    assert_size(k_index);
    return internal_u[k_index];
}
unsigned int& quad_index::operator[](unsigned int k_index)
{
    assert_size(k_index);
    return internal_u[k_index];
}
void quad_index::assert_size(unsigned int k) const
{
    if(k>3)
    {
        throw std::string("Exception ")+std::string(__PRETTY_FUNCTION__)+" "+cpe::string_converter::to_string(k)+">3";
    }
}

/** \brief Get pointer */
const unsigned int* quad_index::pointer() const
{
    return &internal_u[0];
}

}
