/*
**    TP CPE Lyon
**    Copyright (C) 2014 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#ifndef SURFACE_PARAM_DRAWABLE_HPP
#define SURFACE_PARAM_DRAWABLE_HPP

#include "surface_param.hpp"
#include "../lib3d/vec3.hpp"
#include "../libspline/quad_index.hpp"

namespace cpe
{



/** \brief Wrapper class to help drawing a surface_param class */
class surface_param_drawable : public surface_param
{
    public:

        // ********************************************* //
        // ********************************************* //
        //  CONSTRUCTORS
        // ********************************************* //
        // ********************************************* //

        /** \brief empty constructor */
        surface_param_drawable();
        /** \brief direct constructor with size */
        surface_param_drawable(unsigned int Nu,unsigned int Nv);

        // ********************************************* //
        // ********************************************* //
        //  Update drawing parameters
        // ********************************************* //
        // ********************************************* //

        /** \brief update connectivity of needed */
        void update_connectivity();

        /** \brief set a specific normal */
        void set_normal(unsigned int ku,unsigned int kv,const cpe::vec3& n);


        // ********************************************* //
        // ********************************************* //
        //  Get drawing pointers
        // ********************************************* //
        // ********************************************* //

        /** \brief get pointer on connectivity */
        const unsigned int* pointer_draw_connectivity() const;
        /** \brief get pointer on normal */
        const float* pointer_draw_normal() const;
        /** \brief get pointer on vertices */
        const float* pointer_draw_vertices() const;





    private:

        /** \brief force to build connectivity */
        void build_connectivity();

        /** \brief internal connectivity */
        std::vector<quad_index> internal_connectivity;
        /** \brief internal normal */
        std::vector<vec3> internal_normal;

        /** \brief storage for size u stored since the previous update */
        unsigned int internal_old_size_u;
        /** \brief storage for size u stored since the previous update */
        unsigned int internal_old_size_v;
};
}

#endif
