/*
**    TP CPE Lyon
**    Copyright (C) 2014 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/** SCENE HEADER */

#pragma once


#ifndef SCENE_HPP
#define SCENE_HPP

#include "../lib3d/vec3.hpp"
#include "../libmesh/mesh.hpp"
#include "../libspline/grid.hpp"
#include "../libspline/surface_param_drawable.hpp"
#include "../libinterface/picking_data.hpp"
#include "../libinterface/gui_interface.hpp"

#include <vector>

class glwidget;

class scene
{
    private:

        /** \brief The control grid for the spline */
        cpe::grid grid_1;
        /** \brief The Spline surface */
        cpe::surface_param_drawable surf_1;



    public:

        // ********************************************* //
        // ********************************************* //
        //  CONSTRUCTORS
        // ********************************************* //
        // ********************************************* //

        /** \brief Empty constructor for the scene */
        scene();


        // ********************************************* //
        // ********************************************* //
        //  Scene construction and drawing
        // ********************************************* //
        // ********************************************* //

        /** \brief Method called only once at the beginning (load files ...) */
        void load_model();
        /** \brief Method called at every frame */
        void draw_scene();



        // ********************************************* //
        // ********************************************* //
        //  Gui Helpers
        // ********************************************* //
        // ********************************************* //


        /** \brief Set the GLWidget pointer */
        void set_glwidget(glwidget *p_widget);

        /** \brief get the gui helper class */
        cpe::gui_interface& gui_visible_interface();



    private:


        /** \brief storage for the sphere (to draw the grid position) */
        cpe::mesh sphere;
        /** \brief Init the sphere loading */
        void preload_sphere();


        /** \brief Storage class for visibility modified in the gui */
        cpe::gui_interface gui_visible;
        /** \brief Init the visibility gui parameters */
        void init_gui_visibility();


};

#endif
