/*
**    TP CPE Lyon
**    Copyright (C) 2014 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/



#include "grid.hpp"
#include "../libinterface/string_helper.hpp"
#include "patch4.hpp"
#include <fstream>
#include <cmath>


namespace cpe
{

grid::grid()
    :internal_size_u(0),internal_size_v(0)
{}

grid::grid(unsigned int Nu,unsigned int Nv)
    :internal_size_u(Nu),internal_size_v(Nv)
{
    internal_data.resize(Nu*Nv);
}

void grid::resize(unsigned int Nu,unsigned int Nv)
{
    internal_size_u=Nu;
    internal_size_v=Nv;
    internal_data.resize(Nu*Nv);
}

unsigned int grid::size_u() const
{
    return internal_size_u;
}
unsigned int grid::size_v() const
{
    return internal_size_v;
}
const vec3& grid::operator()(unsigned int ku,unsigned int kv) const
{
    assert_size(ku,kv);
    return internal_data[ku+internal_size_u*kv];
}
vec3& grid::operator()(unsigned int ku,unsigned int kv)
{
    assert_size(ku,kv);
    return internal_data[ku+internal_size_u*kv];
}
void grid::assert_size(unsigned int ku,unsigned int kv) const
{
    if(ku>=internal_size_u || kv>=internal_size_v)
    {
        throw std::string("Exception ")+std::string(__PRETTY_FUNCTION__)+" ("+cpe::string_converter::to_string(ku)+","+cpe::string_converter::to_string(kv)+")>("+cpe::string_converter::to_string(internal_size_u)+","+cpe::string_converter::to_string(internal_size_v)+")";
    }
}
patch4 grid::get_patch(unsigned int k_patch_u,unsigned int k_patch_v) const
{
    assert_size(k_patch_u+3,k_patch_v+3);
    return patch4(internal_data[(k_patch_u+0)+internal_size_u*(k_patch_v+0)],internal_data[(k_patch_u+0)+internal_size_u*(k_patch_v+1)],internal_data[(k_patch_u+0)+internal_size_u*(k_patch_v+2)],internal_data[(k_patch_u+0)+internal_size_u*(k_patch_v+3)],
                  internal_data[(k_patch_u+1)+internal_size_u*(k_patch_v+0)],internal_data[(k_patch_u+1)+internal_size_u*(k_patch_v+1)],internal_data[(k_patch_u+1)+internal_size_u*(k_patch_v+2)],internal_data[(k_patch_u+1)+internal_size_u*(k_patch_v+3)],
                  internal_data[(k_patch_u+2)+internal_size_u*(k_patch_v+0)],internal_data[(k_patch_u+2)+internal_size_u*(k_patch_v+1)],internal_data[(k_patch_u+2)+internal_size_u*(k_patch_v+2)],internal_data[(k_patch_u+2)+internal_size_u*(k_patch_v+3)],
                  internal_data[(k_patch_u+3)+internal_size_u*(k_patch_v+0)],internal_data[(k_patch_u+3)+internal_size_u*(k_patch_v+1)],internal_data[(k_patch_u+3)+internal_size_u*(k_patch_v+2)],internal_data[(k_patch_u+3)+internal_size_u*(k_patch_v+3)]);


}

void grid::build_meshgrid()
{
    for(unsigned int ku=0;ku<internal_size_u;++ku)
    {
        for(unsigned int kv=0;kv<internal_size_v;++kv)
        {
            unsigned int index=ku+internal_size_u*kv;
            vec3& p=internal_data[index];
            p.x()=ku;
            p.y()=kv;
        }
    }
}

void grid::build_sphere()
{
    //**********************************//
    //** TO DO: Construire une sphere **//
    //**********************************//

    for(unsigned int ku=0;ku<internal_size_u;++ku)
    {
        for(unsigned int kv=0;kv<internal_size_v;++kv)
        {
            //to do ... construire une sphere
            // internal_data[ku+internal_size_u*kv]=//quelle coordonnee à mettre ?
        }
    }
}

grid& grid::operator*=(float s)
{
    unsigned int N=internal_data.size();
    for(unsigned int k=0;k<N;++k)
        internal_data[k]*=s;
    return *this;
}

grid operator*(float s,const grid& g)
{
    grid g2=g;
    g2*=s;
    return g2;
}

grid operator*(const grid& g,float s)
{
    return s*g;
}

grid& grid::operator+=(const vec3& t)
{
    unsigned int N=internal_data.size();
    for(unsigned int k=0;k<N;++k)
        internal_data[k]+=t;
    return *this;
}

grid& grid::operator-=(const vec3& t)
{
    unsigned int N=internal_data.size();
    for(unsigned int k=0;k<N;++k)
        internal_data[k]-=t;
    return *this;
}

grid operator+(const grid& g,const vec3& t)
{
    grid g2=g;
    g2+=t;
    return g2;
}

grid operator+(const vec3& t,const grid& g)
{
    return g+t;
}

grid operator-(const grid& g,const vec3& t)
{
    grid g2=g;
    g2-=t;
    return g2;
}

grid operator-(const vec3& t,const grid& g)
{
    grid g2=-g;
    g2+=t;
    return g2;
}

grid grid::operator-() const
{
    grid g2=*this;
    g2*=-1;
    return g2;
}

void grid::add_back_u()
{
    if(internal_size_u<2)
        throw std::string("Error in ")+__PRETTY_FUNCTION__+" size must be >2 too add a grid line";

    //copy actual with new size
    grid g2(internal_size_u+1,internal_size_v);
    for(unsigned int ku=0;ku<internal_size_u;++ku)
    {
        for(unsigned int kv=0;kv<internal_size_v;++kv)
        {
            g2(ku,kv)=(*this)(ku,kv);
        }
    }

    //init the new line
    const unsigned int& Nu=internal_size_u;
    for(unsigned int kv=0;kv<internal_size_v;++kv)
        g2(Nu,kv)=2*g2(Nu-1,kv)-g2(Nu-2,kv); //copy the previous direction

    *this=g2;
}

void grid::add_front_u()
{
    if(internal_size_u<2)
        throw std::string("Error in ")+__PRETTY_FUNCTION__+" size must be >2 too add a grid line";

    //copy actual with new size
    grid g2(internal_size_u+1,internal_size_v);
    for(unsigned int ku=0;ku<internal_size_u;++ku)
    {
        for(unsigned int kv=0;kv<internal_size_v;++kv)
        {
            g2(ku+1,kv)=(*this)(ku,kv);
        }
    }

    //init the new line
    for(unsigned int kv=0;kv<internal_size_v;++kv)
        g2(0,kv)=2*g2(1,kv)-g2(2,kv); //copy the previous direction

    *this=g2;
}

void grid::add_back_v()
{
    if(internal_size_v<2)
        throw std::string("Error in ")+__PRETTY_FUNCTION__+" size must be >2 too add a grid line";

    //copy actual with new size
    grid g2(internal_size_u,internal_size_v+1);
    for(unsigned int ku=0;ku<internal_size_u;++ku)
    {
        for(unsigned int kv=0;kv<internal_size_v;++kv)
        {
            g2(ku,kv)=(*this)(ku,kv);
        }
    }

    //init the new line
    const unsigned int& Nv=internal_size_v;
    for(unsigned int ku=0;ku<internal_size_u;++ku)
        g2(ku,Nv)=2*g2(ku,Nv-1)-g2(ku,Nv-2); //copy the previous direction

    *this=g2;
}

void grid::add_front_v()
{
    if(internal_size_v<2)
        throw std::string("Error in ")+__PRETTY_FUNCTION__+" size must be >2 too add a grid line";

    //copy actual with new size
    grid g2(internal_size_u,internal_size_v+1);
    for(unsigned int ku=0;ku<internal_size_u;++ku)
    {
        for(unsigned int kv=0;kv<internal_size_v;++kv)
        {
            g2(ku,kv+1)=(*this)(ku,kv);
        }
    }

    //init the new line
    for(unsigned int ku=0;ku<internal_size_u;++ku)
        g2(ku,0)=2*g2(ku,1)-g2(ku,2); //copy the previous direction

    *this=g2;
}

void grid::duplicate_boundary()
{
    if(internal_size_u<2 || internal_size_v<2)
        throw std::string("Error in ")+__PRETTY_FUNCTION__+" size must be >2 too add a grid line";

    //copy actual with new size
    grid g2(internal_size_u+2,internal_size_v+2);
    for(unsigned int ku=0;ku<internal_size_u;++ku)
    {
        for(unsigned int kv=0;kv<internal_size_v;++kv)
        {
            g2(ku+1,kv+1)=(*this)(ku,kv);
        }
    }

    //4 borders
    for(unsigned int ku=0;ku<internal_size_u;++ku)
    {
        g2(ku+1,0)=(*this)(ku,0);
        g2(ku+1,internal_size_v+1)=(*this)(ku,internal_size_v-1);
    }
    for(unsigned int kv=0;kv<internal_size_v;++kv)
    {
        g2(0,kv+1)=(*this)(0,kv);
        g2(internal_size_u+1,kv+1)=(*this)(internal_size_u-1,kv);
    }

    //4 corners
    g2(0,0)=(*this)(0,0);
    g2(internal_size_u+1,0)=(*this)(internal_size_u-1,0);
    g2(internal_size_u+1,internal_size_v+1)=(*this)(internal_size_u-1,internal_size_v-1);
    g2(0,internal_size_v+1)=(*this)(0,internal_size_v-1);

    *this=g2;
}

void grid::suppress_back_u()
{

    if(internal_size_u<=4 || internal_size_v<4)
        throw std::string("Error in ")+__PRETTY_FUNCTION__+" size must be >4 too suppress a grid line";

    //copy actual with new size
    grid g2(internal_size_u-1,internal_size_v);
    for(unsigned int ku=0;ku<internal_size_u-1;++ku)
    {
        for(unsigned int kv=0;kv<internal_size_v;++kv)
        {
            g2(ku,kv)=(*this)(ku,kv);
        }
    }

    *this=g2;
}

void grid::suppress_front_u()
{
    if(internal_size_u<=4 || internal_size_v<4)
        throw std::string("Error in ")+__PRETTY_FUNCTION__+" size must be >4 too suppress a grid line";

    //copy actual with new size
    grid g2(internal_size_u-1,internal_size_v);
    for(unsigned int ku=1;ku<internal_size_u;++ku)
    {
        for(unsigned int kv=0;kv<internal_size_v;++kv)
        {
            g2(ku-1,kv)=(*this)(ku,kv);
        }
    }

    *this=g2;
}

void grid::suppress_back_v()
{
    if(internal_size_u<4 || internal_size_v<=4)
        throw std::string("Error in ")+__PRETTY_FUNCTION__+" size must be >2 too suppress a grid line";

    //copy actual with new size
    grid g2(internal_size_u,internal_size_v-1);
    for(unsigned int ku=0;ku<internal_size_u;++ku)
    {
        for(unsigned int kv=0;kv<internal_size_v-1;++kv)
        {
            g2(ku,kv)=(*this)(ku,kv);
        }
    }

    *this=g2;
}

void grid::suppress_front_v()
{
    if(internal_size_u<4 || internal_size_v<=4)
        throw std::string("Error in ")+__PRETTY_FUNCTION__+" size must be >2 too suppress a grid line";

    //copy actual with new size
    grid g2(internal_size_u,internal_size_v-1);
    for(unsigned int ku=0;ku<internal_size_u;++ku)
    {
        for(unsigned int kv=+1;kv<internal_size_v;++kv)
        {
            g2(ku,kv-1)=(*this)(ku,kv);
        }
    }

    *this=g2;
}

void grid::write(const std::string& filename) const
{
    std::ofstream stream(filename.c_str());
    if(stream.good()!=true)
        throw std::string("Cannot open file")+filename+" in "+__PRETTY_FUNCTION__;

    stream<<internal_size_u<<" "<<internal_size_v<<std::endl;
    for(unsigned int ku=0;ku<internal_size_u;++ku)
    {
        for(unsigned int kv=0;kv<internal_size_v;++kv)
        {
            const vec3& p=internal_data[ku+internal_size_u*kv];
            stream<<p.x()<<" "<<p.y()<<" "<<p.z()<<std::endl;
        }
        stream<<std::endl;
    }

    stream.close();
}

grid grid::read(const std::string& filename)
{


    std::ifstream stream(filename.c_str());
    if(stream.good()!=true)
        throw std::string("Cannot open file")+filename+" in "+__PRETTY_FUNCTION__;

    std::string buffer;


    unsigned int Nu,Nv;
    stream>>Nu;
    stream>>Nv;

    grid new_grid(Nu,Nv);

    for(unsigned int ku=0;ku<Nu;++ku)
    {
        for(unsigned int kv=0;kv<Nv;++kv)
        {
            double x=0,y=0,z=0;
            stream>>x;
            stream>>y;
            stream>>z;

            new_grid(ku,kv)=vec3(x,y,z);
        }
    }

    stream.close();


    return new_grid;
}

}
