/*
**    TP CPE Lyon
**    Copyright (C) 2014 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#ifndef PATCH4_HPP
#define PATCH4_HPP

#include "../lib3d/vec3.hpp"

namespace cpe
{
class matrix4;

class patch4
{
    public:

        // ********************************************* //
        // ********************************************* //
        //  CONSTRUCTORS
        // ********************************************* //
        // ********************************************* //

        /** \brief Empty constructor */
        patch4();
        /** \brief Direct constructor */
        patch4(const vec3& x00,const vec3& x01,const vec3& x02,const vec3& x03,
               const vec3& x10,const vec3& x11,const vec3& x12,const vec3& x13,
               const vec3& x20,const vec3& x21,const vec3& x22,const vec3& x23,
               const vec3& x30,const vec3& x31,const vec3& x32,const vec3& x33);

        // ********************************************* //
        // ********************************************* //
        //  Data
        // ********************************************* //
        // ********************************************* //

        /** \brief Get value */
        const vec3& operator()(unsigned int ku ,unsigned int kv) const;
        /** \brief Get value */
        vec3& operator()(unsigned int ku ,unsigned int kv);

        /** \brief Get x coordinate in matrix4 form */
        matrix4 get_x() const;
        /** \brief Get y coordinate in matrix4 form */
        matrix4 get_y() const;
        /** \brief Get z coordinate in matrix4 form */
        matrix4 get_z() const;

    private:

        /** Ensure (ku,kv) is within [[0,3]] */
        void assert_size(unsigned int ku,unsigned int kv) const;

        /** \brief internal data */
        vec3 internal_data[16];
};
}

#endif
