/*
**    TP CPE Lyon
**    Copyright (C) 2014 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#pragma once

#ifndef SURFACE_PARAM_HPP
#define SURFACE_PARAM_HPP

#include "../lib3d/vec3.hpp"
#include <vector>

namespace cpe
{
class surface_param
{
    public:

        // ********************************************* //
        // ********************************************* //
        //  CONSTRUCTORS
        // ********************************************* //
        // ********************************************* //

        /** \brief empty constructor */
        surface_param();
        /** \brief direct constructor */
        surface_param(unsigned int Nu,unsigned int Nv);

        // ********************************************* //
        // ********************************************* //
        //  Size
        // ********************************************* //
        // ********************************************* //


        /** \brief resize the grid */
        void resize(unsigned int Nu,unsigned int Nv);

        /** \brief return the size in u direction */
        unsigned int size_u() const;
        /** \brief return the size in v direction */
        unsigned int size_v() const;


        // ********************************************* //
        // ********************************************* //
        //  Data
        // ********************************************* //
        // ********************************************* //

        /** \brief Accessor to the value (ku,kv) */
        const vec3& operator()(unsigned int ku,unsigned int kv) const;
        /** \brief Accessor to the value (ku,kv) */
        vec3& operator()(unsigned int ku,unsigned int kv);


        // ********************************************* //
        // ********************************************* //
        //  Internal access
        // ********************************************* //
        // ********************************************* //

        /** \brief export internal pointer */
        const vec3* pointer() const;
        /** \brief export internal pointer unprotected, use with care */
        vec3* pointer_unprotected();


    protected:

        /** \brief getter fast but unprotected for heritage */
        const vec3& get(unsigned int ku,unsigned int kv) const;

        /** \brief ensure (ku,kv) is coherent with the size of the grid */
        void assert_size(unsigned int ku,unsigned int kv) const;

    private:





        /** \brief internal size in u */
        unsigned int internal_size_u;
        /** \brief internal size in v */
        unsigned int internal_size_v;
        /** \brief internal data */
        std::vector<vec3> internal_data;
};
}

#endif
