/*
**    TP CPE Lyon
**    Copyright (C) 2014 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "surface_param_drawable.hpp"


namespace cpe
{
surface_param_drawable::surface_param_drawable()
    :surface_param(),internal_old_size_u(0),internal_old_size_v(0)
{}

surface_param_drawable::surface_param_drawable(unsigned int Nu,unsigned int Nv)
    :surface_param(Nu,Nv),internal_old_size_u(0),internal_old_size_v(0)
{
    build_connectivity();
}

void surface_param_drawable::update_connectivity()
{
    const unsigned int& Nu=size_u();
    const unsigned int& Nv=size_v();

    if(Nu!=internal_old_size_u || Nv!=internal_old_size_v)
        build_connectivity();

}

void surface_param_drawable::set_normal(unsigned int ku,unsigned int kv,const cpe::vec3& n)
{
    const unsigned int& Nu=size_u();
    const unsigned int& Nv=size_v();
    if(internal_normal.size()!=Nu*Nv)
        internal_normal.resize(Nu*Nv);

    assert_size(ku,kv);
    internal_normal[ku+Nu*kv]=n;

}



const unsigned int* surface_param_drawable::pointer_draw_connectivity() const
{

    return internal_connectivity[0].pointer();
}

const float* surface_param_drawable::pointer_draw_normal() const
{
    return internal_normal[0].pointer();
}

const float* surface_param_drawable::pointer_draw_vertices() const
{
    return this->pointer()->pointer();
}

void surface_param_drawable::build_connectivity()
{

    const unsigned int& Nu=size_u();
    const unsigned int& Nv=size_v();


    if(internal_connectivity.size()!=(Nu-1)*(Nv-1))
        internal_connectivity.resize((Nu-1)*(Nv-1));

    for(unsigned int ku=0;ku<Nu-1;++ku)
        for(unsigned int kv=0;kv<Nv-1;++kv)
            internal_connectivity[ku+(Nu-1)*kv]=quad_index((ku+0)+Nu*(kv+0),(ku+1)+Nu*(kv+0),(ku+1)+Nu*(kv+1),(ku+0)+Nu*(kv+1));

    internal_old_size_u=Nu;
    internal_old_size_v=Nv;
}

}
